@@ -1,19 +1,11 @@
-use std::borrow::Cow;
-
+use atuin_common::api::{ErrorResponse, IndexResponse};
 use axum::{response::IntoResponse, Json};
-use serde::{Deserialize, Serialize};
 
 pub mod history;
 pub mod user;
 
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 
-#[derive(Debug, Serialize, Deserialize)]
-pub struct IndexResponse {
-    pub homage: String,
-    pub version: String,
-}
-
 pub async fn index() -> Json<IndexResponse> {
     let homage = r#""Through the fathomless deeps of space swims the star turtle Great A'Tuin, bearing on its back the four giant elephants who carry on their shoulders the mass of the Discworld." -- Sir Terry Pratchett"#;
 
@@ -23,11 +15,6 @@ pub async fn index() -> Json<IndexResponse> {
     })
 }
 
-#[derive(Debug, Serialize, Deserialize)]
-pub struct ErrorResponse<'a> {
-    pub reason: Cow<'a, str>,
-}
-
 impl<'a> IntoResponse for ErrorResponseStatus<'a> {
     fn into_response(self) -> axum::response::Response {
         (self.status, Json(self.error)).into_response()
@@ -39,15 +26,20 @@ pub struct ErrorResponseStatus<'a> {
     pub status: http::StatusCode,
 }
 
-impl<'a> ErrorResponse<'a> {
-    pub fn with_status(self, status: http::StatusCode) -> ErrorResponseStatus<'a> {
+pub trait RespExt<'a> {
+    fn with_status(self, status: http::StatusCode) -> ErrorResponseStatus<'a>;
+    fn reply(reason: &'a str) -> Self;
+}
+
+impl<'a> RespExt<'a> for ErrorResponse<'a> {
+    fn with_status(self, status: http::StatusCode) -> ErrorResponseStatus<'a> {
         ErrorResponseStatus {
             error: self,
             status,
         }
     }
 
-    pub fn reply(reason: &'a str) -> ErrorResponse {
+    fn reply(reason: &'a str) -> ErrorResponse {
         Self {
             reason: reason.into(),
         }
