@@ -1,6 +1,9 @@
 use std::borrow::Borrow;
 
-use axum::{extract::Path, Extension, Json};
+use axum::{
+    extract::{Path, State},
+    Extension, Json,
+};
 use http::StatusCode;
 use sodiumoxide::crypto::pwhash::argon2id13;
 use tracing::{debug, error, instrument};
@@ -8,8 +11,9 @@ use uuid::Uuid;
 
 use super::{ErrorResponse, ErrorResponseStatus, RespExt};
 use crate::{
-    database::{Database, Postgres},
+    database::Database,
     models::{NewSession, NewUser},
+    router::AppState,
     settings::Settings,
 };
 
@@ -30,10 +34,11 @@ pub fn verify_str(secret: &str, verify: &str) -> bool {
 }
 
 #[instrument(skip_all, fields(user.username = username.as_str()))]
-pub async fn get(
+pub async fn get<DB: Database>(
     Path(username): Path<String>,
-    db: Extension<Postgres>,
+    state: State<AppState<DB>>,
 ) -> Result<Json<UserResponse>, ErrorResponseStatus<'static>> {
+    let db = &state.0.database;
     let user = match db.get_user(username.as_ref()).await {
         Ok(user) => user,
         Err(sqlx::Error::RowNotFound) => {
@@ -53,10 +58,10 @@ pub async fn get(
 }
 
 #[instrument(skip_all)]
-pub async fn register(
-    Json(register): Json<RegisterRequest>,
+pub async fn register<DB: Database>(
     settings: Extension<Settings>,
-    db: Extension<Postgres>,
+    state: State<AppState<DB>>,
+    Json(register): Json<RegisterRequest>,
 ) -> Result<Json<RegisterResponse>, ErrorResponseStatus<'static>> {
     if !settings.open_registration {
         return Err(
@@ -73,6 +78,7 @@ pub async fn register(
         password: hashed,
     };
 
+    let db = &state.0.database;
     let user_id = match db.add_user(&new_user).await {
         Ok(id) => id,
         Err(e) => {
@@ -101,10 +107,11 @@ pub async fn register(
 }
 
 #[instrument(skip_all, fields(user.username = login.username.as_str()))]
-pub async fn login(
+pub async fn login<DB: Database>(
+    state: State<AppState<DB>>,
     login: Json<LoginRequest>,
-    db: Extension<Postgres>,
 ) -> Result<Json<LoginResponse>, ErrorResponseStatus<'static>> {
+    let db = &state.0.database;
     let user = match db.get_user(login.username.borrow()).await {
         Ok(u) => u,
         Err(sqlx::Error::RowNotFound) => {
