@@ -1,36 +1,31 @@
 use async_trait::async_trait;
 use axum::{
-    extract::{FromRequest, RequestParts},
-    handler::Handler,
+    extract::FromRequestParts,
     response::IntoResponse,
     routing::{get, post},
-    Extension, Router,
+    Router,
 };
 use eyre::Result;
+use http::request::Parts;
 use tower::ServiceBuilder;
 use tower_http::trace::TraceLayer;
 
-use super::{
-    database::{Database, Postgres},
-    handlers,
-};
+use super::{database::Database, handlers};
 use crate::{models::User, settings::Settings};
 
 #[async_trait]
-impl<B> FromRequest<B> for User
+impl<DB: Send + Sync> FromRequestParts<AppState<DB>> for User
 where
-    B: Send,
+    DB: Database,
 {
     type Rejection = http::StatusCode;
 
-    async fn from_request(req: &mut RequestParts<B>) -> Result<Self, Self::Rejection> {
-        let postgres = req
-            .extensions()
-            .get::<Postgres>()
-            .ok_or(http::StatusCode::INTERNAL_SERVER_ERROR)?;
-
+    async fn from_request_parts(
+        req: &mut Parts,
+        state: &AppState<DB>,
+    ) -> Result<Self, Self::Rejection> {
         let auth_header = req
-            .headers()
+            .headers
             .get(http::header::AUTHORIZATION)
             .ok_or(http::StatusCode::FORBIDDEN)?;
         let auth_header = auth_header
@@ -44,7 +39,8 @@ where
             return Err(http::StatusCode::FORBIDDEN);
         }
 
-        let user = postgres
+        let user = state
+            .database
             .get_session_user(token)
             .await
             .map_err(|_| http::StatusCode::FORBIDDEN)?;
@@ -56,7 +52,17 @@ where
 async fn teapot() -> impl IntoResponse {
     (http::StatusCode::IM_A_TEAPOT, "☕")
 }
-pub fn router(postgres: Postgres, settings: Settings) -> Router {
+
+#[derive(Clone)]
+pub struct AppState<DB> {
+    pub database: DB,
+    pub settings: Settings,
+}
+
+pub fn router<DB: Database + Clone + Send + Sync + 'static>(
+    database: DB,
+    settings: Settings,
+) -> Router {
     let routes = Router::new()
         .route("/", get(handlers::index))
         .route("/sync/count", get(handlers::history::count))
@@ -73,11 +79,7 @@ pub fn router(postgres: Postgres, settings: Settings) -> Router {
     } else {
         Router::new().nest(path, routes)
     }
-    .fallback(teapot.into_service())
-    .layer(
-        ServiceBuilder::new()
-            .layer(TraceLayer::new_for_http())
-            .layer(Extension(postgres))
-            .layer(Extension(settings)),
-    )
+    .fallback(teapot)
+    .with_state(AppState { database, settings })
+    .layer(ServiceBuilder::new().layer(TraceLayer::new_for_http()))
 }
