@@ -4,6 +4,7 @@ use clap::Subcommand;
 use eyre::{Result, WrapErr};
 
 use atuin_client::{database::Sqlite, settings::Settings};
+use env_logger::Builder;
 
 #[cfg(feature = "sync")]
 mod sync;
@@ -38,9 +39,12 @@ pub enum Cmd {
 impl Cmd {
     #[tokio::main(flavor = "current_thread")]
     pub async fn run(self) -> Result<()> {
-        pretty_env_logger::init();
+        Builder::new()
+            .filter_level(log::LevelFilter::Off)
+            .parse_env("ATUIN_LOG")
+            .init();
 
-        let settings = Settings::new().wrap_err("could not load client settings")?;
+        let mut settings = Settings::new().wrap_err("could not load client settings")?;
 
         let db_path = PathBuf::from(settings.db_path.as_str());
         let mut db = Sqlite::new(db_path).await?;
@@ -49,7 +53,7 @@ impl Cmd {
             Self::History(history) => history.run(&settings, &mut db).await,
             Self::Import(import) => import.run(&mut db).await,
             Self::Stats(stats) => stats.run(&mut db, &settings).await,
-            Self::Search(search) => search.run(&mut db, &settings).await,
+            Self::Search(search) => search.run(&mut db, &mut settings).await,
             #[cfg(feature = "sync")]
             Self::Sync(sync) => sync.run(settings, &mut db).await,
         }
